@@ -15,14 +15,14 @@
 #include "sync.h"
 #include "rectangle.h"
 
-#ifdef USE_STD_STRING
-#include <string>
-#endif
-
 BEGIN_GIGABASE_NAMESPACE
 
 #ifndef dbDatabaseOffsetBits
-#define dbDatabaseOffsetBits 32 // 37 - up to 1 terabyte
+#ifdef LARGE_DATABASE_SUPPORT
+#define dbDatabaseOffsetBits 40 // up to 1 terabyte
+#else
+#define dbDatabaseOffsetBits 32 // 37 - 128Gb, 40 - up to 1 terabyte
+#endif
 #endif
 
 #ifndef dbDatabaseOidBits
@@ -33,7 +33,7 @@ BEGIN_GIGABASE_NAMESPACE
  * Object indentifier type
  */
 #if dbDatabaseOidBits > 32
-typedef nat8 oid_t;  // It will work only for 64-bit OS
+typedef nat8 oid_t;
 #else
 typedef nat4 oid_t;
 #endif
@@ -42,41 +42,56 @@ typedef nat4 oid_t;
  * Object offset in the file type
  */
 #if dbDatabaseOffsetBits > 32
-typedef nat8 offs_t; // It will work only for 64-bit OS
+typedef nat8 offs_t;
+typedef db_int8 soffs_t;
 #else
 typedef nat4 offs_t;
+typedef int4 soffs_t;
 #endif
 
+typedef oid_t cardinality_t;
+
 #include "selection.h"
 
 /**
  * Types of field index
  */
 enum dbIndexType {
-    HASHED  = 1, // hash table
-    INDEXED = 2, // B-tree
-    CASE_INSENSITIVE = 4, // Index is case insensetive
+    HASHED  = 1,                   // hash table
+    INDEXED = 2,                   // B-tree
+    CASE_INSENSITIVE = 4,          // Index is case insensitive
 
     DB_FIELD_CASCADE_DELETE = 8,   // Used by OWNER macro, do not set it explicitly
-    UNIQUE = 16, // should be used in conjunction with HASHED or INDEXED - unique constraint 
-
-    AUTOINCREMENT = 32, // field is assigned automaticall incremented value
+    UNIQUE = 16,                   // should be used in conjunction with HASHED or INDEXED - unique constraint 
 
+    AUTOINCREMENT = 32,            // field is assigned automaticall incremented value
+    OPTIMIZE_DUPLICATES = 64,      // index with lot of duplicate key values
+    DB_BLOB_CASCADE_DELETE = 128,  // Used by BLOB macro to mark BLOB fields which should be deleted together with 
+                                   // containing them record
+    DB_TIMESTAMP = 256,            // field is used as timestamp (this flag is used by CLI to perfrom proper mapping,
+                                   // it is not used by C++ API)
     DB_FIELD_INHERITED_MASK = ~(HASHED|INDEXED)
 };
 
 /**
- * Macro for describing indexed fields
+ * Macros for describing indexed fields
  */
 #define KEY(x, index) \
-    *dbDescribeField(new dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
-                                           sizeof(x), index), x)
+    *GB_NS::dbDescribeField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+                                                         sizeof(x), index), x)
+
+#define KEY_WITH_DEFAULT(x, index, defaultValue) \
+	*GB_NS::dbDescribeField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+	sizeof(x), index), x, defaultValue)
 
 /**
- * Macro for describing non-indexed fields
+ * Macros for describing non-indexed fields
  */
 #define FIELD(x) KEY(x, 0)
 
+#define FIELD_WITH_DEFAULT(x, defaultValue) KEY_WITH_DEF(x, 0, defaultValue)
+
+
 /**
  * Comparator for user defined raw binary fields
  */
@@ -86,8 +101,8 @@ typedef int (*dbUDTComparator)(void*, void*, size_t);
  * Macro used to describe indexed raw binary fields with user-defined comparator
  */
 #define UDT(x, index, comparator) \
-    *dbDescribeRawField(new dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
-                                                sizeof(x), index), (dbUDTComparator)comparator)
+    *GB_NS::dbDescribeRawField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+                                                            sizeof(x), index), (GB_NS::dbUDTComparator)comparator)
 
 /**
  * Macro used to describe raw binary field
@@ -106,8 +121,8 @@ typedef int (*dbUDTComparator)(void*, void*, size_t);
  * and <code>inverse</code> - field in the referenced table contining inverse reference.
  */
 #define RELATION(x,inverse) \
-    *dbDescribeField(new dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
-                                             sizeof(x), 0, #inverse), x)
+    *GB_NS::dbDescribeField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+                                                         sizeof(x), 0, STRLITERAL(#inverse)), x)
 
 /**
  * Macro for describing relations between two tables.  
@@ -115,8 +130,8 @@ typedef int (*dbUDTComparator)(void*, void*, size_t);
  * and <code>inverse</code> - field in the referenced table contining inverse reference.
  */
 #define INDEXED_RELATION(x,inverse) \
-    *dbDescribeField(new dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
-                                             sizeof(x), INDEXED, #inverse), x)
+    *GB_NS::dbDescribeField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+                                                         sizeof(x), GB_NS::INDEXED, STRLITERAL(#inverse)), x)
  
 /**
  * Macro used to define relation owner (when owner is deleted, all referenced
@@ -124,16 +139,25 @@ typedef int (*dbUDTComparator)(void*, void*, size_t);
  * <code>RELATION</code> macro to describe relation with owner.  
  */
 #define OWNER(x,member) \
-    *dbDescribeField(new dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
-                                             sizeof(x), DB_FIELD_CASCADE_DELETE, \
-                                             #member), x)
+    *GB_NS::dbDescribeField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+                                                         sizeof(x), GB_NS::DB_FIELD_CASCADE_DELETE, \
+                                                         STRLITERAL(#member)), x)
 
 /**
  * Macro used to describe method of the class which can be invoked from SubSQL
  */
 #define METHOD(x) \
-    *dbDescribeMethod(new dbFieldDescriptor(STRLITERAL(#x)), &self::x)
+    *GB_NS::dbDescribeMethod(new GB_NS::dbFieldDescriptor(STRLITERAL(#x)), &self::x)
 
+/**
+ * Macro for describing BLOB fields belonging only to one record. Actually BLOB in GigaBASE is implemeted as reference
+ * to BLOB body. It is possible to refer the same BLOB from several records. That is why when record is removed, 
+ * BLOBs referenced by this record are not removed. But in most typical case BLOB belongs only to one record.
+ * And it will be convinient of its removed togehter with this record. To describe such BLOB fields BLOB macro should be used.
+ */
+#define BLOB(x) \
+    *GB_NS::dbDescribeField(new GB_NS::dbFieldDescriptor(STRLITERAL(#x), (char*)&x-(char*)this, \
+                                                         sizeof(x), GB_NS::DB_BLOB_CASCADE_DELETE), x)
 /**
  * Macro used to describe superclass for this class
  */
@@ -145,10 +169,10 @@ typedef int (*dbUDTComparator)(void*, void*, size_t);
  * macros separated by comma inside this macro to describe all fields of the record
  */
 #define TYPE_DESCRIPTOR(fields) \
-    dbFieldDescriptor* dbDescribeComponents(dbFieldDescriptor*) { \
+    GB_NS::dbFieldDescriptor* dbDescribeComponents(GB_NS::dbFieldDescriptor*) { \
         return &fields; \
     } \
-    static dbTableDescriptor dbDescriptor
+    static GB_NS::dbTableDescriptor dbDescriptor
 
 /**
  * Macro used to describe class, the only difference from <code>TYPE_DESCRIPTOR</code>
@@ -157,50 +181,87 @@ typedef int (*dbUDTComparator)(void*, void*, size_t);
  */
 #define CLASS_DESCRIPTOR(name, fields) \
     typedef name self; \
-    dbFieldDescriptor* dbDescribeComponents(dbFieldDescriptor*) { \
+    GB_NS::dbFieldDescriptor* dbDescribeComponents(GB_NS::dbFieldDescriptor*) { \
         return &fields; \
     } \
-    static dbTableDescriptor dbDescriptor
+    static GB_NS::dbTableDescriptor dbDescriptor
 
-#if defined(_MSC_VER) && _MSC_VER+0 <= 1100 || defined(__MWERKS__)
 /**
  * Register table descriptor and assign it to specified database
  */
-#define REGISTER_IN(table, database) \
-    template <> dbTableDescriptor* dbGetTableDescriptor<table>(table*) \
-      { return &table::dbDescriptor; }              \
-    static dbFieldDescriptor* dbDescribeComponentsOf##table() \
-     { return ((table*)0)->dbDescribeComponents(NULL); }     \
-    dbTableDescriptor table::dbDescriptor(_T(#table), database, sizeof(table), \
-                                          &dbDescribeComponentsOf##table)
+#if (defined(_MSC_VER) && _MSC_VER+0 < 1200) || defined(__MWERKS__)
+    #if defined(_MSC_VER)
+        #define GET_TABLE_DESC_PREFIX
+    #else
+        #define GET_TABLE_DESC_PREFIX template<>
+    #endif
+    #define GET_TABLE_DESC_PARAM(T) <T>(T*)
 #else
-/**
- * Register table descriptor and assign it to specified database
- */
-#define REGISTER_IN(table, database) \
-    dbTableDescriptor* dbGetTableDescriptor(table*) \
-      { return &table::dbDescriptor; }              \
-    static dbFieldDescriptor* dbDescribeComponentsOf##table() \
-      { return ((table*)0)->dbDescribeComponents(NULL); }     \
-    dbTableDescriptor table::dbDescriptor(_T(#table), database, sizeof(table), \
-                                          &dbDescribeComponentsOf##table)
+    #define GET_TABLE_DESC_PREFIX
+    #define GET_TABLE_DESC_PARAM(T) (T*)
 #endif
 
+#if defined(_MSC_VER) && _MSC_VER+0 <= 1300
+    #define TABLE_DESC_PREFIX 
+#else
+    #define TABLE_DESC_PREFIX template<>
+#endif
+
+#define REGISTER_IN(table, database) \
+    GET_TABLE_DESC_PREFIX GB_NS::dbTableDescriptor* dbGetTableDescriptor GET_TABLE_DESC_PARAM(table) \
+      { return &table::dbDescriptor; }              \
+    static GB_NS::dbFieldDescriptor* dbDescribeComponentsOf##table() \
+     { return ((table*)0)->dbDescribeComponents(NULL); }     \
+    GB_NS::dbTableDescriptor table::dbDescriptor(_T(#table), database, sizeof(table), \
+                                                 &dbDescribeComponentsOf##table)
+
+#define REGISTER_TEMPLATE_IN(table, database) \
+    static GB_NS::dbFieldDescriptor* dbDescribeComponentsOf##table() \
+     { return ((table*)0)->dbDescribeComponents(NULL); } \
+    TABLE_DESC_PREFIX GB_NS::dbTableDescriptor table::dbDescriptor(_T(#table), database, sizeof(table), \
+                                                            &dbDescribeComponentsOf##table); \
+    GET_TABLE_DESC_PREFIX GB_NS::dbTableDescriptor* dbGetTableDescriptor GET_TABLE_DESC_PARAM(table) \
+      { return &table::dbDescriptor; }              \
+
+#define REGISTER_IN_NS(ns, table, database)                              \
+    GET_TABLE_DESC_PREFIX GB_NS::dbTableDescriptor* dbGetTableDescriptor GET_TABLE_DESC_PARAM(ns::table) \
+    { return &ns::table::dbDescriptor; }                                \
+    static GB_NS::dbFieldDescriptor* dbDescribeComponentsOf##ns##__##table() \
+    { return ((ns::table*)0)->dbDescribeComponents(NULL); }             \
+    GB_NS::dbTableDescriptor ns::table::dbDescriptor(_T(ns##__##table), database, sizeof(ns::table), \
+                                                     &dbDescribeComponentsOf##ns##__##table)
+
+#define REGISTER_TEMPLATE_IN_NS(ns, table, database)                              \
+    GET_TABLE_DESC_PREFIX GB_NS::dbTableDescriptor* dbGetTableDescriptor GET_TABLE_DESC_PARAM(ns::table) \
+    { return &ns::table::dbDescriptor; }                                \
+    static GB_NS::dbFieldDescriptor* dbDescribeComponentsOf##ns##__##table() \
+    { return ((ns::table*)0)->dbDescribeComponents(NULL); }             \
+    TABLE_DESC_PREFIX GB_NS::dbTableDescriptor ns::table::dbDescriptor(_T(ns##__##table), database, sizeof(ns::table), \
+                                                                &dbDescribeComponentsOf##ns##__##table)
+
+
 /**
- * Register table descripttor. It will be assigned to the database when database will be 
+ * Register table descriptor. It will be assigned to the database when database will be 
  * opened
  */
 #define REGISTER(table) REGISTER_IN(table, NULL)
+#define REGISTER_TEMPLATE(table) REGISTER_TEMPLATE_IN(table, NULL)
+#define REGISTER_NS(ns, table) REGISTER_IN_NS(ns, table, NULL)
+#define REGISTER_TEMPLATE_NS(ns, table) REGISTER_TEMPLATE_IN_NS(ns, table, NULL)
 
 /**
  * Register database and mark it as unsigned. Programmer should explicitly
  * specify database in all operations.
  */
-#define DETACHED_TABLE ((dbDatabase*)-1)
+#define DETACHED_TABLE ((GB_NS::dbDatabase*)-1)
 #define REGISTER_UNASSIGNED(table) REGISTER_IN(table, DETACHED_TABLE)
+#define REGISTER_TEMPLATE_UNASSIGNED(table) REGISTER_TEMPLATE_IN(table, DETACHED_TABLE)
+#define REGISTER_UNASSIGNED_NS(ns, table) REGISTER_IN_NS(ns, table, DETACHED_TABLE)
+#define REGISTER_TEMPLATE_UNASSIGNED_NS(ns, table) REGISTER_TEMPLATE_IN_NS(ns, table, DETACHED_TABLE)
 
 
 class dbDatabase;
+class dbSession;
 class dbAnyArray;
 class dbTableDescriptor;
 class dbAnyMethodTrampoline;
@@ -296,6 +357,15 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
      */
     int                indexType;
 
+    /**
+     * Default value of the field (in case an existing scheme is extended)
+     */
+    union {		
+        bool Bool;		
+        int1 Int1;		int2 Int2;		int4 Int4;		db_int8 Db_Int8;
+        real4 Real4;	real8 Real8;
+    } defaultValue;
+    
     /** 
      * Offset to the field in database
      */
@@ -349,7 +419,8 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
         ComponentOfArray   = 0x01,
         HasArrayComponents = 0x02,
         OneToOneMapping    = 0x04,
-        Updated            = 0x08
+        Updated            = 0x08,
+        HasArrayOfArrayComponents = 0x10
     };
     int                attr;
 
@@ -436,17 +507,24 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
     size_t copyRecordExceptOneField(dbFieldDescriptor* field,
                                     byte* dst, byte* src, size_t offs);
 
+    
+    enum StoreMode {
+        Insert,
+        Update,
+        Import
+    };
+
     /**
      * Store record fields in the databases
      * This method performs interation thtough all components in one scope
      * and recursively invoke itself for structure and array components.      
      * @param dst place in the database where record should be stored
      * @param src pointer to the application object
-     * @param offset to the end of varying part
-     * @param insert flag used to distringuish update fro insert (needed for autoincremented fields)
+     * @param offs offset to the end of varying part
+     * @param mode Insert, Update or Import mode
      * @return size of the record
      */     
-    size_t storeRecordFields(byte* dst, byte* src, size_t offs, bool insert);
+    size_t storeRecordFields(byte* dst, byte* src, size_t offs, StoreMode mode);
 
     /**
      * Mask updated fields.
@@ -457,6 +535,15 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
      */
     void markUpdatedFields(byte* dst, byte* src);
 
+    /**
+     * Mask updated fields.
+     * This method performs interation thtough all components in one scope
+     * and recursively invoke itself for structure and array components.      
+     * @param dst old image of the record in the database
+     * @param src new image of the record in the database
+     */
+    void markUpdatedFields2(byte* dst, byte* src);
+
     /**
      * Fetch record from the database
      * This method performs interation thtough all components in one scope
@@ -466,6 +553,13 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
      */
      void fetchRecordFields(byte* dst, byte* src);
 
+    /**
+     * Find component with specified symbol name (for structures only)
+     * @param name component symbol name
+     * @return descriptor of the field or <code>NULL</code> if not found
+     */
+    dbFieldDescriptor* findSymbol(const char_t* name);
+
     /**
      * Find component with specified name (for structures only)
      * @param name component name
@@ -475,7 +569,7 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
 
     /**
      * Get first component of the field (for structures only)
-     * @param return first component of the structure
+     * @return first component of the structure
      */
     dbFieldDescriptor* getFirstComponent() { 
         return components;
@@ -513,7 +607,7 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
     /**
      * Adjust offsets within application objects for descriptors of base classes.
      */
-    dbFieldDescriptor& adjustOffsets(long offs);
+    dbFieldDescriptor& adjustOffsets(size_t offs);
 
     /**
      * Field descriptor constructor
@@ -521,18 +615,18 @@ class GIGABASE_DLL_ENTRY dbFieldDescriptor {
      * @param offs offset of the field
      * @param size size of the field
      * @param indexType type of index used for this field
-     * @param name of inverse field
+     * @param inverse name of inverse field
      * @param components comopnents of structure or array
      */
-    dbFieldDescriptor(char_t* name, int offs, int size, int indexType,
-                      char_t* inverse = NULL,
+    dbFieldDescriptor(char_t const* name, size_t offs, size_t size, int indexType,
+                      char_t const* inverse = NULL,
                       dbFieldDescriptor* components = NULL);
 
     /**
      * Constructor of dummy field descriptor 
      * @param  name name of the field
      */
-    dbFieldDescriptor(char_t* name);
+    dbFieldDescriptor(char_t const* name);
 
     /**
      * Field descriptor destructor
@@ -550,20 +644,25 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
     friend class dbReplicatedDatabase;
     friend class dbTable;
     friend class dbAnyCursor;
-    friend class dbSubSql;
     friend class dbParallelQueryContext;
     friend class dbServer;
+    friend class dbSession;
     friend class dbAnyContainer;
     friend class dbColumnBinding;
     friend class dbFieldDescriptor;
     friend class dbSelection;
+    friend class dbTableIterator;
     friend class dbCLI;
+    friend class dbFileTransactionLogger;
+    friend class dbSubSql;
+
   protected:
     /**
      * Chain of all tables in application
      */
     dbTableDescriptor*  next;
     static dbTableDescriptor* chain;
+    static dbMutex*           chainMutex;
 
     /**
      * Chain of all tables associated with database
@@ -633,7 +732,7 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
     dbTableDescriptor*  cloneOf;
 
     /**
-     * Size of tghe correspondent applciation object
+     * Size of tghe correspondent application object
      */
     size_t              appSize;
 
@@ -687,14 +786,28 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
      */
     dbSelection               batch;
 
+    /**
+     * ID of transaction when this table descriptor was last refreshed
+     */
+    int                       transactionId;
 
 
+    /**
+     * Table attrbiutes
+     */
+    int                       attr;
+
     /**
      * Function returning list of record fields descriptors
      */
     typedef dbFieldDescriptor* (*describeFunc)();
     describeFunc        describeComponentsFunc;
 
+    /**
+     * Collision chain of table's hash table
+     */
+    dbTableDescriptor*  collisionChain;
+
     /**
      * Calculate total length of all names in table descriptor
      */
@@ -702,7 +815,7 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
 
     /**
      * Recursively set field attributes.
-     * @param fieldList list of record fields
+     * @param fieldsList list of record fields
      * @param prefix prefix for the field (in case of structures or arrays 
      * this functions is invoked resursively for components of this structure or
      * or array
@@ -710,10 +823,13 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
      * @param indexMask index mask for the structore containing the field
      * @param attr attributes of the parent field
      * @return alignment of the field
+     * @param dbsAlignment maximal alignment of the fields in the database 
+     * @param appAlignment maximal alignment of the fields in the application 
      */
-    int calculateFieldsAttributes(dbFieldDescriptor* fieldsList,
+     void calculateFieldsAttributes(dbFieldDescriptor* fieldsList,
                                   char_t const* prefix, int offs,
-                                  int indexMask, int& attr);
+                                  int indexMask, int& attr,
+                                  size_t& dbsAlignment, size_t& appAlignment);
 
     /**
      * Read table definiton from the database and build fields list
@@ -736,6 +852,14 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
      */
     static int initialAutoincrementCount;
 
+
+    /**
+     * Get next table in database
+     */
+    dbTableDescriptor* getNextTable() { 
+        return nextDbTable;
+    }
+
     /**
      * Find field with specified symbol name
      */    
@@ -754,6 +878,13 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
         return columns;
     }
 
+    /**
+     * Get last value of autoincrement counter used for this table
+     * @return last value of autoincrement counter used for this table
+     */
+    int getLastValueOfAutoincrementCount() const { 
+        return autoincrementCount;
+    }
 
     /**
      * Get next field
@@ -777,14 +908,28 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
         return name;
     }
 
+    /**
+     * Get size of instance of the class in an application
+     */
+    size_t size() { 
+        return appSize;
+    }
+
+    /**
+     * Get table identifier
+     */
+    oid_t getId() { 
+        return tableId;
+    }
 
     /**
      * Check whether table descriptor in the database is the same as
      * table appplication table descriptor
      * @param table database table descriptor
+     * @param ignoreIndices ignore presence of indices during comparison
      * @return <code>true</code> if two table descriptors are equal
      */
-     bool equal(dbTable* table);
+    bool equal(dbTable* table, bool ignoreIndices = false);
 
     /**
      * Check whether fprmats of table descriptor in the database 
@@ -792,16 +937,18 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
      * information for performing conversion of record to new format
      * @param table database table descriptor
      * @param confirmDeleteColumns whether deletion of columns in allowed from non empty table
+     * @param preserveExistedIndices preserve existed indices
+     * @param isEmpty table is empty
      * @return <code>true</code> if no reformatting is needed
      */
-    bool match(dbTable* table, bool confirmDeleteColumns);
+    bool match(dbTable* table, bool confirmDeleteColumns, bool preserveExistedIndices, bool isEmpty);
 
     /**
      * Check consuistency of declared realations (check that referenced table 
      * actually contains declared inverse reference field). 
      * This method also resolve references between table.
      */
-    void checkRelationship();
+    bool checkRelationship();
 
     /**
      * Get reference to associated database
@@ -821,6 +968,9 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
      */
     void setFlags();
 
+    static dbMutex& getChainMutex();
+    void link();
+    void unlink();
 
     /**
      * Remove all table descriptors except static ones
@@ -842,7 +992,7 @@ class GIGABASE_DLL_ENTRY dbTableDescriptor {
      * @param func function returninglist of field descriptors
      * @param original roiginal table descriptor (for cloned descriptors)
      */
-    dbTableDescriptor(char_t*            tableName, 
+    dbTableDescriptor(char_t const*      tableName, 
                       dbDatabase*        db, 
                       size_t             objSize,
                       describeFunc       func,
@@ -889,6 +1039,7 @@ struct dbField {
         tpStructure,
         tpRawBinary,
         tpStdString,
+        tpMfcString,
         tpRectangle,
         tpUnknown
     };
@@ -911,7 +1062,17 @@ struct dbField {
     /**
      * Field type: one of <code>dbField::FieldTypes</code> constants
      */
+#ifdef OLD_FIELD_DESCRIPTOR_FORMAT
     int4      type;
+#else 
+#if BYTE_ORDER == BIG_ENDIAN
+    int4      flags : 24;
+    int4      type  : 8;
+#else
+    int4      type  : 8;
+    int4      flags : 24;
+#endif
+#endif
 
     /**
      *  Offset of the field in the record
@@ -929,7 +1090,7 @@ struct dbField {
     oid_t     hashTable;
 
     /**
-     * B-Tree for field indexed by means of T-Ttree
+     * B-Tree for field indexed by means of B-Ttree
      */
     oid_t     bTree;
 };
@@ -1012,59 +1173,88 @@ inline dbFieldDescriptor* dbDescribeRawField(dbFieldDescriptor* fd, dbUDTCompara
     return fd;
 }
 
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, int1&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, int1&, int1 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt1;
+	fd->defaultValue.Int1 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, int2&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, int2&, int2 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt2;
+	fd->defaultValue.Int2 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, int4&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, int4&, int4 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt4;
+	fd->defaultValue.Int4 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, db_int8&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, db_int8&, db_int8 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt8;
+	fd->defaultValue.Db_Int8 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat1&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat1&, nat1 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt1;
+	fd->defaultValue.Int1 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat2&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat2&, nat2 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt2;
+	fd->defaultValue.Int2 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat4&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat4&, nat4 defaultValue = 0)
 {
-    fd->type = fd->appType = dbField::tpInt4;
-    return fd;
+	fd->type = fd->appType = dbField::tpInt4;
+	fd->defaultValue.Int4 = defaultValue;
+	return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat8&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, nat8&, nat8 defaultValue = 0)
 {
     fd->type = fd->appType = dbField::tpInt8;
+	fd->defaultValue.Db_Int8 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, bool&)
+
+#if SIZEOF_LONG != 8 
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, long&, long defaultValue = 0)
+{
+    fd->type = fd->appType = sizeof(long) == 4 ? dbField::tpInt4 : dbField::tpInt8;
+    if (sizeof(long) == 4) fd->defaultValue.Int4 = defaultValue;
+    else fd->defaultValue.Db_Int8 = defaultValue;
+    return fd;
+}
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, unsigned long&, unsigned long defaultValue = 0)
+{
+    fd->type = fd->appType = sizeof(long) == 4 ? dbField::tpInt4 : dbField::tpInt8;
+    if (sizeof(long) == 4) fd->defaultValue.Int4 = defaultValue;
+    else fd->defaultValue.Db_Int8 = defaultValue;
+    return fd;
+}
+#endif
+
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, bool&, bool defaultValue = false)
 {
     fd->type = fd->appType = dbField::tpBool;
+	fd->defaultValue.Bool = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, real4&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, real4&, real4 defaultValue = 0.0)
 {
     fd->type = fd->appType = dbField::tpReal4;
+	fd->defaultValue.Real4 = defaultValue;
     return fd;
 }
-inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, real8&)
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, real8&, real8 defaultValue = 0.0)
 {
     fd->type = fd->appType = dbField::tpReal8;
+	fd->defaultValue.Real8 = defaultValue;
     return fd;
 }
 inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, rectangle&)
@@ -1087,6 +1277,19 @@ inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, STD_STRING&)
     return fd;
 }
 #endif
+#ifdef USE_MFC_STRING
+inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, MFC_STRING&)
+{
+    fd->type = dbField::tpString;
+    fd->appType = dbField::tpMfcString;
+    fd->dbsSize = sizeof(dbVarying);
+    fd->alignment = 4;
+    fd->components = new dbFieldDescriptor(STRLITERAL("[]"));
+    fd->components->type = fd->components->appType = dbField::tpInt1 + (sizeof(char_t) - 1);
+    fd->components->dbsSize = fd->components->appSize = fd->components->alignment = sizeof(char_t); 
+    return fd;
+}
+#endif
 
 inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, char_t const*&)
 {
@@ -1114,13 +1317,13 @@ template<class T>
 inline dbFieldDescriptor* dbDescribeField(dbFieldDescriptor* fd, T& x)
 {
     fd->type = fd->appType = dbField::tpStructure;
-    fd->components = x.dbDescribeComponents(fd);
+    fd->components = x.dbDescribeComponents(fd);	
     return fd;
 }
 
 
 /** 
- * Trampolinefor invocation of methods from SubSQL
+ * Trampoline for invocation of methods from SubSQL
  */
 class GIGABASE_DLL_ENTRY dbAnyMethodTrampoline {
   public:
@@ -1151,10 +1354,12 @@ class GIGABASE_DLL_ENTRY dbAnyMethodTrampoline {
      * Trampoline desctructor
      */
     virtual~dbAnyMethodTrampoline();
+
+    void* operator new(size_t size);
+    void  operator delete(void* p);
 };
 
-#if defined(__APPLE__) || defined(__VACPP_MULTI__) || defined(__IBMCPP__) || \
-    (__SUNPRO_CC >= 0x520 && __SUNPRO_CC_COMPAT == 5)
+#if defined(__APPLE__) || defined(__VACPP_MULTI__) || defined(__IBMCPP__) || defined(__HP_aCC) || (defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x510 && __SUNPRO_CC_COMPAT == 5)
 /**
  * Template for method trampoline implementation
  */
@@ -1172,7 +1377,7 @@ class dbMethodTrampoline : public dbAnyMethodTrampoline {
             *(R*)result = (((T*)(data + this->cls->dbsOffs))->*method)();
         } else { 
             T rec;
-            cls->components->fetchRecordFields((byte*)&rec, data);
+            this->cls->components->fetchRecordFields((byte*)&rec, data);
             *(R*)result = (rec.*method)();
         }
     }
@@ -1182,7 +1387,12 @@ class dbMethodTrampoline : public dbAnyMethodTrampoline {
     }
 
     dbMethodTrampoline(dbFieldDescriptor* fd, mfunc f)
-    : dbAnyMethodTrampoline(fd), method(f), cls(fd), optimized(false) {}
+    : dbAnyMethodTrampoline(fd)
+    {
+        this->method = f;
+        this->cls = fd;
+        this->optimized = false;
+    }
 };
 
 #else
@@ -1198,7 +1408,7 @@ class dbMethodTrampoline : public dbAnyMethodTrampoline {
 
     void invoke(byte* data, void* result) {
         T rec;
-        cls->components->fetchRecordFields((byte*)&rec, data);
+        this->cls->components->fetchRecordFields((byte*)&rec, data);
         *(R*)result = (rec.*method)();
     }
     dbAnyMethodTrampoline* optimize();
@@ -1208,13 +1418,18 @@ class dbMethodTrampoline : public dbAnyMethodTrampoline {
 };
 
 template<class T, class R>
-class dbMethodFastTrampoline : public dbMethodTrampoline<T,R> {
+class dbMethodFastTrampoline : public dbAnyMethodTrampoline {
+    typedef R (T::*mfunc)();
+    mfunc method;
   public:
-    void invoke(byte* data, void* result) {
-        *(R*)result = (((T*)(data + cls->dbsOffs))->*method)();
+    dbAnyMethodTrampoline* optimize() { 
+        return this;
     }
-    dbMethodFastTrampoline(dbMethodTrampoline<T,R>* mt)
-    : dbMethodTrampoline<T,R>(mt->cls, mt->method) {
+    void invoke(byte* data, void* result) {
+        *(R*)result = (((T*)(data + this->cls->dbsOffs))->*method)();
+    }
+    dbMethodFastTrampoline(dbMethodTrampoline<T,R>* mt) 
+    : dbAnyMethodTrampoline(mt->cls), method(mt->method) {
         delete mt;
     }
 };
