@@ -33,11 +33,11 @@ class GIGABASE_DLL_ENTRY dbAnyContainer : public dbAnyReference {
   protected:
     dbFieldDescriptor* fd;
 
-    void create(dbDatabase& db, bool caseInsensitive);
-    void purge(dbDatabase& db);
-    void free(dbDatabase& db);
-    void add(dbDatabase& db, dbAnyReference const& ref);
-    void remove(dbDatabase& db, dbAnyReference const& ref);
+    void create(dbDatabase* db, bool caseInsensitive = false, bool thick = false, bool unique = false);
+    void purge(dbDatabase* db);
+    void free(dbDatabase* db);
+    bool add(dbDatabase* db, dbAnyReference const& ref);
+    void remove(dbDatabase* db, dbAnyReference const& ref);
     int  search(dbAnyCursor& cursor, void const* from, void const* till, bool ascent = true);
     int  spatialSearch(dbAnyCursor& cursor, rectangle const& r, SpatialSearchType type);
 
@@ -56,6 +56,7 @@ class dbContainer : public dbAnyContainer {
      * @param cursor cursor to iterate through selected resords
      * @param from inclusive low bound for the search key, if <code>NULL</code> then there is no low bound
      * @param till inclusive high bound for the search key,  if <code>NULL</code> then there is no high bound
+     * @param ascent key order: <code>true</code> - ascending order, <code>false</code> - descending order
      * @return number of selected records
      */
     int search(dbCursor<T>& cursor, void const* from, void const* till, bool ascent = true) {
@@ -64,7 +65,7 @@ class dbContainer : public dbAnyContainer {
     /**
      * Select records with sepcified value of the key
      * @param cursor cursor to iterate through selected resords
-     * @param key searched value of the key
+     * @param key searched key
      * @return number of selected records
      */    
     int search(dbCursor<T>& cursor, void const* key) {
@@ -74,6 +75,7 @@ class dbContainer : public dbAnyContainer {
     /**
      * Select all records in the container
      * @param cursor cursor to iterate through selected resords
+     * @param ascent cursor cursor to iterate through selected resords
      * @return number of selected records
      */
     int search(dbCursor<T>& cursor, bool ascent = true) {
@@ -97,21 +99,21 @@ class dbContainer : public dbAnyContainer {
      * @param caseInsensitive case insensitive flag (used only for string field)
      */
     void create(bool caseInsensitive = false) {
-        dbAnyContainer::create(T::dbDescriptor.db, caseInsensitive);
+        dbAnyContainer::create(T::dbDescriptor.getDatabase(), caseInsensitive);
     }
 
     /**
      * Clear the container
      */
     void purge() {
-        dbAnyContainer::purge(T::dbDescriptor.db);
+        dbAnyContainer::purge(T::dbDescriptor.getDatabase());
     }
 
     /**
      * Deallocate container
      */
     void free() {
-        dbAnyContainer::free(T::dbDescriptor.db);
+        dbAnyContainer::free(T::dbDescriptor.getDatabase());
     }
 
     /**
@@ -119,7 +121,7 @@ class dbContainer : public dbAnyContainer {
      * @param ref reference to the record added to the container
      */
     void add(dbReference<T> const& ref) {
-        dbAnyContainer::add(T::dbDescriptor.db, ref);
+        dbAnyContainer::add(T::dbDescriptor.getDatabase(), ref);
     }
 
     /**
@@ -127,7 +129,7 @@ class dbContainer : public dbAnyContainer {
      * @param ref reference to the record deleted from the container
      */
     void remove(dbReference<T> const& ref) {
-        dbAnyContainer::remove(T::dbDescriptor.db, ref);
+        dbAnyContainer::remove(T::dbDescriptor.getDatabase(), ref);
     }
 
     /**
@@ -135,6 +137,56 @@ class dbContainer : public dbAnyContainer {
      * @param fieldName name of the key field used by container
      */
     dbContainer(const char_t* fieldName) : dbAnyContainer(fieldName, T::dbDescriptor) {}
+    
+    /**
+     * Create new container in specified database.
+     * @param db database
+     */
+    void create(dbDatabase* db) {
+        dbAnyContainer::create(db);
+    }
+
+    /**
+     * Clear the container in specified database.
+     * @param db database
+     */
+    void purge(dbDatabase* db) {
+        dbAnyContainer::purge(db);
+    }
+
+    /**
+     * Deallocate container in specified database.
+     * @param db database
+     */
+    void free(dbDatabase* db) {
+        dbAnyContainer::free(db);
+    }
+
+    /**
+     * Add new record to container in specified database.
+     * @param db database
+     * @param ref reference to the record added to the container
+     */
+    void add(dbDatabase* db, dbReference<T> const& ref) {
+        dbAnyContainer::add(db, ref);
+    }
+
+    /**
+     * Remove record from the container in specified database.
+     * @param db database
+     * @param ref reference to the record deleted from the container
+     */
+    void remove(dbDatabase* db, dbReference<T> const& ref) {
+        dbAnyContainer::remove(db, ref);
+    }
+
+    /**
+     * Constructor of the contanier reference
+     * @param db database
+     * @param fieldName name of the key field used by container
+     */
+    dbContainer(dbDatabase* db, const char_t* fieldName) 
+    : dbAnyContainer(fieldName, *db->lookupTable(&T::dbDescriptor)) {}
 };
 
 END_GIGABASE_NAMESPACE
