@@ -11,15 +11,54 @@
 #ifndef __CURSOR_H__
 #define __CURSOR_H__
 
-#include "selection.h"
+#include "btree.h"
+#include "rtree.h"
 
 BEGIN_GIGABASE_NAMESPACE
 
+#include "selection.h"
+
 enum dbCursorType {
     dbCursorViewOnly,
-    dbCursorForUpdate
+    dbCursorForUpdate,
+    /**
+     * Execute query incrementally
+     */
+    dbCursorIncremental,
+    /** Detached cursor commits transaction immediately after execution of the query,
+     *  and then temporarily start new transaction for fetching each element.
+     *  Such cursor should be used in conjunction with DO_NOT_REUSE_OID_WITHIN_SESSION to detect 
+     *  removed objects.
+     *  Please notice that this cursor is static - it only executed from the selection removed objects,
+     *  but doesn't try to add the selection new records which match search criteria or remove 
+     *  record which do not match this criteria any more.
+     */
+    dbCursorDetached
 };
 
+/**
+ *  Iterator through table records
+ */
+class dbTableIterator : public dbAbstractIterator {
+    dbAnyCursor* cursor;
+    dbExprNode*  filter;
+    oid_t        curr;
+
+  public:
+    void init(dbAnyCursor* cursor, dbExprNode*  filter) {
+        this->cursor = cursor;
+        this->filter = filter;
+        curr = 0;
+    }
+
+    virtual oid_t next();
+    virtual oid_t prev();
+    virtual oid_t first();
+    virtual oid_t last();
+};
+
+
+
 /**
  * Base class for all cursors
  */
@@ -28,18 +67,22 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
     friend class dbHashTable;
     friend class dbRtreePage;
     friend class dbBtreePage;
-    friend class dbBtreeLeafPage;
+    friend class dbRtreeIterator;
+    friend class dbBtreeIterator;
+    friend class dbTableIterator;
+    friend class dbThickBtreePage;
     friend class dbSubSql;
     friend class dbStatement;
     friend class dbServer;
     friend class dbAnyContainer;
     friend class dbCLI;
+    friend class JniResultSet;
   public:
     /**
      * Get number of selected records
      * @return number of selected records
      */
-    int getNumberOfRecords() { return selection.nRows; }
+    cardinality_t getNumberOfRecords() const { return selection.nRows; }
 
     /**
      * Remove current record
@@ -50,16 +93,33 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Checks whether selection is empty
      * @return true if there is no current record
      */
-    bool isEmpty() { 
-	return currId == 0; 
+    bool isEmpty() const { 
+        return currId == 0; 
+    }
+
+
+    /**
+     * Get current reocrd OID
+     * @return current record OID or 0 if thereis no current record
+     */
+    oid_t getOid() {
+        return currId;
+    }
+
+    /**
+     * Check whether this cursor can be used for update
+     * @return true if it is update cursor
+     */
+    bool isUpdateCursor() const { 
+        return type == dbCursorForUpdate;
     }
 
     /**
      * Checks whether limit for number of selected reacord is reached
      * @return true if limit is reached
      */
-    bool isLimitReached() { 
-	return selection.nRows >= limit || selection.nRows >= stmtLimitLen; 
+    bool isLimitReached() const { 
+        return selection.nRows >= limit || selection.nRows >= stmtLimitLen; 
     }
 
     /**
@@ -72,17 +132,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * fill local structure and pass pointer to it to select method.
      * @return number of selected records
      */
-    int select(dbQuery& query, dbCursorType aType, void* paramStruct = NULL) {
-        paramBase = paramStruct;
-        type = aType;
-        reset();
-        db->select(this, query);
-        paramBase = NULL;
-        if (gotoFirst() && prefetch) {
-            fetch();
-        }
-        return selection.nRows;
-    }
+    cardinality_t select(dbQuery& query, dbCursorType aType, void* paramStruct = NULL);
 
     /**
      * Extract OIDs of selected recrods in array
@@ -91,7 +141,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      *  If <code>arr</code> is null, then new array is created by  new oid_t[] and returned by this method
      * @return if <code>arr</code> is not null, then <code>arr</code>, otherwise array created by this method
      */
-    oid_t* toArrayOfOid(oid_t* arr);
+    oid_t* toArrayOfOid(oid_t* arr) const;
 
     /**
      * Execute query with default cursor type.
@@ -99,7 +149,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * @param paramStruct pointer to structure with parameters.
      * @return number of selected records
      */    
-    int select(dbQuery& query, void* paramStruct = NULL) {
+    cardinality_t select(dbQuery& query, void* paramStruct = NULL) {
         return select(query, defaultType, paramStruct);
     }
 
@@ -110,7 +160,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * @param paramStruct pointer to structure with parameters.
      * @return number of selected records
      */
-    int select(char_t const* condition, dbCursorType aType, void* paramStruct = NULL) {
+    cardinality_t select(char_t const* condition, dbCursorType aType, void* paramStruct = NULL) {
         dbQuery query(condition);
         return select(query, aType, paramStruct);
     }
@@ -121,7 +171,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * @param paramStruct pointer to structure with parameters.
      * @return number of selected records
      */    
-    int select(char_t const* condition, void* paramStruct = NULL) {
+    cardinality_t select(char_t const* condition, void* paramStruct = NULL) {
         return select(condition, defaultType, paramStruct);
     }
 
@@ -130,7 +180,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * @param aType cursor type: <code>dbCursorForUpdate, dbCursorViewOnly</code>
      * @return number of selected records
      */    
-    int select(dbCursorType aType) {
+    cardinality_t select(dbCursorType aType) {
         type = aType;
         reset();
         db->select(this);
@@ -144,7 +194,7 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Select all records from the table with default cursor type
      * @return number of selected records
      */    
-    int select() {
+    cardinality_t select() {
         return select(defaultType);
     }
 
@@ -154,26 +204,49 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * @param value searched value of the key
      * @return number of selected records
      */    
-    int selectByKey(char_t const* key, void const* value);
+    cardinality_t selectByKey(char_t const* key, void const* value);
+
+    /**
+     * Select all records from the table with specfied value of the key
+     * @param field key field
+     * @param value searched value of the key
+     * @return number of selected records
+     */    
+    cardinality_t selectByKey(dbFieldDescriptor* field, void const* value);
 
     /**
      * Select all records from the table with specfied range of the key values
      * @param key name of the key field
      * @param minValue inclusive low bound for key values, if <code>NULL</code> then there is no low bound
      * @param maxValue inclusive high bound for key values, if <code>NULL</code> then there is no high bound
+     * @param ascent key order: <code>true</code> - ascending order, <code>false</code> - descending order
      * @return number of selected records
      */    
-    int selectByKeyRange(char_t const* key, void const* minValue, void const* maxValue, bool ascent = true);
+    cardinality_t selectByKeyRange(char_t const* key, void const* minValue, void const* maxValue, bool ascent = true);
+
+    /**
+     * Select all records from the table with specfied range of the key values
+     * @param field key field
+     * @param minValue inclusive low bound for key values, if <code>NULL</code> then there is no low bound
+     * @param maxValue inclusive high bound for key values, if <code>NULL</code> then there is no high bound
+     * @param ascent key order: <code>true</code> - ascending order, <code>false</code> - descending order
+     * @return number of selected records
+     */    
+    cardinality_t selectByKeyRange(dbFieldDescriptor* field, void const* minValue, void const* maxValue, bool ascent = true);
 
     /**
      * Update current record. You should changed value of current record before and then call
-     * update method to save changes to the database
+     * update method to save changes to the database.
+     * @return true if record was successfully updated, false if update cause unique constraint violation
      */
-    void update() {
-        assert(type == dbCursorForUpdate && currId != 0);
-        updateInProgress = true;
-        db->update(currId, table, record);
-        updateInProgress = false;
+    bool update() {
+        if (type != dbCursorForUpdate) {
+            db->handleError(dbDatabase::CursorError, "Readonly cursor");
+        }
+        if (currId == 0) {
+            db->handleError(dbDatabase::CursorError, "No current record");
+        }
+        return db->update(currId, table, record);
     }
 
     /**
@@ -208,6 +281,36 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      */
     void setPrefetchMode(bool mode) { prefetch = mode; }
 
+    /**
+     * Check if cursor is incremental. Incremental cursor is used only when type is dbCursorIncremental.
+     * But even if the type of the cursor is dbCursorIncremental, the cursor 
+     * may not always be incremental - it depends on a query. 
+     * Queries containing order by clause except ordering by index key, can not be executed incrementally.
+     * In case of incremental cursor, select() returns 0 if no record is selected or 1 
+     * if selection is not empty,
+     * but precise number of selected records is not reported since it is not known.
+     * @return whether cursor is incremental or not.
+     */
+    bool isIncremental() { 
+        return iterator != NULL;
+    }
+
+    /**
+     * Check if cursor has dbIncrementalType.
+     * @return whether cursor has incremental hint or not
+     */
+    bool hasIncrementalHint() { 
+        return type == dbCursorIncremental;
+    }
+
+    /**
+     * Enable or disable duplicates checking (if programmer knows that disjuncts in query do not intersect, then
+     * he can disable duplicate checking and avoid bitmap allocation
+     */
+    void enableCheckForDuplicates(bool enabled) {
+        checkForDuplicatedIsEnabled = enabled;
+    }
+ 
     /**
      * Reset cursor
      */
@@ -217,13 +320,13 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      * Check whether current record is the last one in the selection
      * @return true if next() method will return <code>NULL</code>
      */
-    bool isLast(); 
+    bool isLast() const; 
 
     /**
      * Check whether current record is the first one in the selection
      * @return true if prev() method will return <code>NULL</code>
      */
-    bool isFirst(); 
+    bool isFirst() const; 
 
     /**
      * Freeze cursor. This method makes it possible to save current state of cursor, close transaction to allow
@@ -246,12 +349,47 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
      */
     bool skip(int n);
 
+    /**
+     * Position cursor on the record with the specified OID
+     * @param oid object identifier of record
+     * @return poistion of the record in the selection or -1 if record with such OID is not in selection
+     */
+    int seek(oid_t oid);
+
     /**
      * Get descriptor of the table. 
      * @return descriptor of the table associated with the cursor
      */
     dbTableDescriptor* getTable() { return table; }
 
+
+    /**
+     * Check if record with specified OID is in selection
+     * @return <code>true</code> if record with such OID was selected
+     */
+    bool isInSelection(oid_t oid);
+
+    /**
+     * Fetch current record.
+     * You should use this method only if prefetch mode is disabled 
+     */
+    void fetch() {
+        dbRecord* row = (type == dbCursorDetached) ? db->fetchRow(tie, currId) : db->getRow(tie, currId);
+        table->columns->fetchRecordFields(record, (byte*)row);
+    }
+
+    /**
+     * Check if there is more records in the selection
+     */
+    bool hasNext() const;
+
+    /**
+     * Check if there is current record in the selection
+     */
+    bool hasCurrent() const { 
+        return currId != 0;
+    }
+
   protected:
     dbDatabase*        db;
     dbTableDescriptor* table;
@@ -267,37 +405,39 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
     dbGetTie           tie;
     void*              paramBase;
 
-    int4*              bitmap; // bitmap to avoid duplicates
-    size_t             bitmapSize;
     bool               eliminateDuplicates;
+    bool               checkForDuplicatedIsEnabled;
     bool               prefetch;
     bool               removed; // current record was removed
-    bool               updateInProgress;
+    bool               lastRecordWasDeleted; //last record was deleted
 
-    size_t             stmtLimitStart;
-    size_t             stmtLimitLen;
-    size_t             nSkipped;
+    cardinality_t      stmtLimitStart;
+    cardinality_t      stmtLimitLen;
+    cardinality_t      nSkipped;
 
-    void checkForDuplicates() {
-        if (!eliminateDuplicates && limit > 1) {
+    dbAbstractIterator*iterator;
+    dbBtreeIterator    btreeIterator;
+    dbRtreeIterator    rtreeIterator;
+    dbTableIterator    tableIterator;
+
+    void checkForDuplicates() { 
+        if (!eliminateDuplicates && checkForDuplicatedIsEnabled && limit > 1) {
             eliminateDuplicates = true;
-            size_t size = (db->currIndexSize + 31) / 32;
-            if (size > bitmapSize) {
-                delete[] bitmap;
-                bitmap = new int4[size];
-                bitmapSize = size;
-            }
-            memset(bitmap, 0, size*4);
+            selection.allocateBitmap(db);
         }
     }
 
     bool isMarked(oid_t oid) {
-        return bitmap != NULL && (bitmap[oid >> 5] & (1 << (oid & 31))) != 0;
+        return selection.bitmap != NULL && (selection.bitmap[(size_t)(oid >> 5)] & (1 << ((int)oid & 31))) != 0;
+    }
+    
+    void deallocateBitmap() {
+        selection.deallocateBitmap();
     }
 
     void mark(oid_t oid) {
-        if (bitmap != NULL) {
-            bitmap[oid >> 5] |= 1 << (oid & 31);
+        if (selection.bitmap != NULL) {
+            selection.bitmap[(size_t)(oid >> 5)] |= 1 << ((int)oid & 31);
         }
     }
 
@@ -317,74 +457,45 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
                 return true;
             }
             if (eliminateDuplicates) {
-                if (bitmap[oid >> 5] & (1 << (oid & 31))) {
+                if (selection.bitmap[(size_t)(oid >> 5)] & (1 << ((int)oid & 31))) {
                     return true;
                 }
-                bitmap[oid >> 5] |= 1 << (oid & 31);
+                selection.bitmap[oid >> 5] |= 1 << (oid & 31);
             }
             selection.add(oid);
             return selection.nRows < limit;
         }
         return false;
     }
+    
+    byte* fetchNext();
+    byte* fetchPrev();
+    byte* fetchFirst();
+    byte* fetchLast();
 
     bool gotoNext();
     bool gotoPrev();
     bool gotoFirst();
     bool gotoLast();
 
-    void setCurrent(dbAnyReference const& ref);
+    bool moveNext();
+    bool movePrev();
 
-    void fetch() {
-        table->columns->fetchRecordFields(record,
-                                          (byte*)db->getRow(tie, currId));
-    }
+    void setCurrent(dbAnyReference const& ref);
 
     void setTable(dbTableDescriptor* aTable) { 
         table = aTable;
         db = aTable->db;
     }
 
-    void setRecord(byte* rec) { 
-        record = rec;
+    void setRecord(void* rec) { 
+        record = (byte*)rec;
     }
 
-    dbAnyCursor(dbTableDescriptor& aTable, dbCursorType aType, byte* rec)
-    : table(&aTable),type(aType),defaultType(aType),
-      allRecords(false),currId(0),record(rec)
-    {
-        limit = dbDefaultSelectionLimit;
-        updateInProgress = false;
-        prefetch = true;
-        removed = false;
-        bitmap = NULL;
-        bitmapSize = 0;
-        eliminateDuplicates = false;
-        db = aTable.db;
-        paramBase = NULL;
-        stmtLimitLen = dbDefaultSelectionLimit;
-        stmtLimitStart = 0;
-        nSkipped = 0;
-    }
+    dbAnyCursor(dbTableDescriptor& aTable, dbCursorType aType, byte* rec);
 
   public:
-    dbAnyCursor() 
-    : table(NULL),type(dbCursorViewOnly),defaultType(dbCursorViewOnly),
-          allRecords(false),currId(0),record(NULL)
-    {
-        limit = dbDefaultSelectionLimit;
-        updateInProgress = false;
-        prefetch = false;
-        removed = false;
-        bitmap = NULL;
-        bitmapSize = 0;
-        eliminateDuplicates = false;
-        db = NULL;
-        paramBase = NULL;
-        stmtLimitLen = dbDefaultSelectionLimit;
-        stmtLimitStart = 0;
-        nSkipped = 0;
-    }
+    dbAnyCursor(dbCursorType aType = dbCursorViewOnly);
     ~dbAnyCursor();
 };
 
@@ -393,6 +504,12 @@ class GIGABASE_DLL_ENTRY dbAnyCursor : public dbL2List {
  */
 template<class T>
 class dbCursor : public dbAnyCursor {
+  private:
+    // Itis not possible to copy cursors
+    dbCursor<T> operator = (dbCursor<T> const& src) { 
+        return *this;
+    } 
+
   protected:
     T record;
 
@@ -407,7 +524,7 @@ class dbCursor : public dbAnyCursor {
     /**
      * Cursor constructor with explicit specification of database.
      * This cursor should be used for unassigned tables. 
-     * @param aDB database in which table lokkup is performed
+     * @param aDb database in which table lokkup is performed
      * @param type cursor type (dbCursorViewOnly by default)
      */
     dbCursor(dbDatabase* aDb, dbCursorType type = dbCursorViewOnly)
@@ -433,11 +550,7 @@ class dbCursor : public dbAnyCursor {
      * @return pointer to the next record or <code>NULL</code> if there is no next record
      */     
     T* next() {
-        if (gotoNext()) {
-            fetch();
-            return &record;
-        }
-        return NULL;
+        return (T*)fetchNext();
     }
 
     /**
@@ -445,11 +558,7 @@ class dbCursor : public dbAnyCursor {
      * @return pointer to the previous record or <code>NULL</code> if there is no previous record
      */     
     T* prev() {
-        if (gotoPrev()) {
-            fetch();
-            return &record;
-        }
-        return NULL;
+        return (T*)fetchPrev();
     }
 
     /**
@@ -457,11 +566,7 @@ class dbCursor : public dbAnyCursor {
      * @return pointer to the first record or <code>NULL</code> if no records were selected
      */
     T* first() {
-        if (gotoFirst()) {
-            fetch();
-            return &record;
-        }
-        return NULL;
+        return (T*)fetchFirst();
     }
 
     /**
@@ -469,19 +574,26 @@ class dbCursor : public dbAnyCursor {
      * @return pointer to the last record or <code>NULL</code> if no records were selected
      */
     T* last() {
-        if (gotoLast()) {
-            fetch();
-            return &record;
-        }
-        return NULL;
+        return (T*)fetchLast();
     }
     
+    /**
+     * Position cursor on the record with the specified OID
+     * @param ref reference to the object
+     * @return position of the record in the selection or -1 if record with such OID is not in selection
+     */
+    int seek(dbReference<T> const& ref) { 
+        return dbAnyCursor::seek(ref.getOid());
+    }
+
     /**
      * Overloaded operator for accessing components of the current record
      * @return pointer to the current record
      */
     T* operator ->() {
-        assert(currId != 0);
+        if (currId == 0) {
+            db->handleError(dbDatabase::CursorError, "No current record");
+        }
         return &record;
     }
 
@@ -497,9 +609,9 @@ class dbCursor : public dbAnyCursor {
 
     /**
      * Get current object idenitifer
-     * @param reference to the current record
+     * @return reference to the current record
      */
-    dbReference<T> currentId() {
+    dbReference<T> currentId() const {
         return dbReference<T>(currId);
     }
     
@@ -507,35 +619,27 @@ class dbCursor : public dbAnyCursor {
      * Convert selection to array of reference
      * @param arr [OUT] array of refeences in which references to selected recrods will be placed
      */
-    void toArray(dbArray< dbReference<T> >& arr) { 
+    void toArray(dbArray< dbReference<T> >& arr) const { 
         arr.resize(selection.nRows);
         toArrayOfOid((oid_t*)arr.base());
     }
 
-    /**
-     * Method nextAvailable allows to iterate through the records in uniform way even when some records 
-     * are removed. For example:
-     * <PRE>
-     * if (cursor.select(q) > 0) { 
-     *     do { 
-     *         if (x) { 
-     *             cursor.remove();
-     *         } else { 
-     *             cursor.update();
-     *         }
-     *     } while (cursor.nextAvaiable());
-     *  }
-     *</PRE>
-     * @return pointer to the current record
-     */     
-    T* nextAvailable() { 
+    T* prevAvailable() { 
         if (!removed) { 
-            return next(); 
-        } else { 
+            return prev(); 
+        } else {
             removed = false;
-            return get();
+            return lastRecordWasDeleted ? get() : prev();
         }
     }
+
+    /**
+     * Check if record with specified OID is in selection
+     * @return <code>true</code> if record with such OID was selected
+     */
+    bool isInSelection(dbReference<T>& ref) {
+        return dbAnyCursor::isInSelection(ref.getOid());
+    }
 };
 
 class dbParallelQueryContext {
