@@ -13,52 +13,59 @@
 
 BEGIN_GIGABASE_NAMESPACE
 
+class dbSession;
+
 /**
  * Element of the query
  */
 class GIGABASE_DLL_ENTRY dbQueryElement {
     friend class dbQuery;
     friend class dbCompiler;
+    friend class dbSession;
     friend class dbQueryExpression;
     friend class dbQueryElementAllocator;
+    friend class dbCLI;
   public:
     enum ElementType {
-	qExpression, // part of SQL expression
-	qVarBool,
-	qVarInt1,
-	qVarInt2,
-	qVarInt4,
-	qVarInt8,
-	qVarReal4,
-	qVarReal8,
-	qVarString,
-	qVarStringPtr,
-	qVarReference,
-	qVarRectangle, 
-	qVarArrayOfRef,
-	qVarArrayOfRefPtr, 
-	qVarRawData,
-#ifdef USE_STD_STRING
-	qVarStdString,
-#endif
-	qVarUnknown
+        qExpression, // part of SQL expression
+        qVarBool,
+        qVarInt1,
+        qVarInt2,
+        qVarInt4,
+        qVarInt8,
+        qVarReal4,
+        qVarReal8,
+        qVarString,
+        qVarStringPtr,
+        qVarReference,
+        qVarRectangle, 
+        qVarRectanglePtr, 
+        qVarArrayOfRef, 
+        qVarArrayOfInt4,
+        qVarArrayOfInt8,
+        qVarArrayOfRefPtr,
+        qVarArrayOfInt4Ptr,
+        qVarArrayOfInt8Ptr,
+        qVarRawData,
+        qVarRawDataPtr,
+        qVarStdString,
+        qVarMfcString,
+        qVarUnknown
     };
 
     void* operator new (size_t size);
-    void  operator delete(void* p) { 
-	delete[] (char*)p;
-    }
+    void  operator delete(void* p);
 
-    char* dump(char* buf);
-    char* dumpValues(char* buf);
+    char_t* dump(char_t* buf);
+    char_t* dumpValues(char_t* buf);
 
     dbQueryElement(ElementType t, void const* p,
-		   dbTableDescriptor* table = NULL)
+                   dbTableDescriptor* table = NULL)
     {
-	type = t;
-	ptr  = p;
-	ref  = table;
-	next = NULL;
+        type = t;
+        ptr  = p;
+        ref  = table;
+        next = NULL;
     }
   private:
     dbQueryElement*    next;
@@ -75,27 +82,27 @@ class GIGABASE_DLL_ENTRY dbQueryElement {
  */
 class GIGABASE_DLL_ENTRY dbQueryElementAllocator { 
     friend class dbDatabase;
-    dbMutex&        mutex;
+    dbMutex         mutex;
     dbQueryElement* freeChain;
     
   public:
     void free(dbQueryElement* first, dbQueryElement** lastNext) { 
-	dbCriticalSection cs(mutex);
-	if (first != NULL) { 
-	    *lastNext = freeChain;
-	    freeChain = first;
-	}
+        dbCriticalSection cs(mutex);
+        if (first != NULL) { 
+            *lastNext = freeChain;
+            freeChain = first;
+        }
     }
-	
+        
     void* allocate(size_t size) { 
-	dbCriticalSection cs(mutex);
-	dbQueryElement* elem = freeChain;
-	if (elem != NULL) {
-	    freeChain = elem->next;
-	    return elem;
-	} else {
-	    return new byte[size];
-	}
+        dbCriticalSection cs(mutex);
+        dbQueryElement* elem = freeChain;
+        if (elem != NULL) {
+            freeChain = elem->next;
+            return elem;
+        } else {
+            return dbMalloc(size);
+        }
     }
     dbQueryElementAllocator();
     ~dbQueryElementAllocator();
@@ -103,9 +110,6 @@ class GIGABASE_DLL_ENTRY dbQueryElementAllocator {
     static dbQueryElementAllocator instance;
 };
 
-inline void* dbQueryElement::operator new(size_t size) {
-    return dbQueryElementAllocator::instance.allocate(size);
-}
 
 /**
  * This class represents component of the structure.
@@ -114,9 +118,9 @@ inline void* dbQueryElement::operator new(size_t size) {
  * This class defines its own comparison methods:
  * <PRE>
  *     dbQueryExpression operator == (char const* field) { 
- *	   dbQueryExpression expr;
- *	   expr = dbComponent(field,"jday"),"=",jday;
- *    	   return expr;
+ *         dbQueryExpression expr;
+ *         expr = dbComponent(field,"jday"),"=",jday;
+ *         return expr;
  *     }
  * </PRE>
  * Making it possible to sepcify queries like this (assume that record has column "released" with type dbDate:
@@ -148,14 +152,14 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
 
   public:
     dbQueryExpression& add(dbQueryElement::ElementType type, void const* ptr, dbTableDescriptor* table = NULL) {
-	last = &(*last = new dbQueryElement(type, ptr, table))->next;
-	operand = (type == dbQueryElement::qExpression);
-	return *this;
+        last = &(*last = new dbQueryElement(type, ptr, table))->next;
+        operand = (type == dbQueryElement::qExpression);
+        return *this;
     }
 
     dbQueryExpression& operator = (char_t const* ptr) {
-	first = NULL, last = &first;
-	return add(dbQueryElement::qExpression, ptr);
+        first = NULL, last = &first;
+        return add(dbQueryElement::qExpression, ptr);
     }
     dbQueryExpression& operator = (dbComponent const& comp);
 
@@ -173,6 +177,26 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
     dbQueryExpression& operator,(db_int8 const& ptr) {
         return add(dbQueryElement::qVarInt8, &ptr);
     }
+    dbQueryExpression& operator,(nat1 const& ptr) {
+        return add(dbQueryElement::qVarInt1, &ptr);
+    }
+    dbQueryExpression& operator,(nat2 const& ptr) {
+        return add(dbQueryElement::qVarInt2, &ptr);
+    }
+    dbQueryExpression& operator,(nat4 const& ptr) {
+        return add(dbQueryElement::qVarInt4, &ptr);
+    }
+    dbQueryExpression& operator,(db_nat8 const& ptr) {
+        return add(dbQueryElement::qVarInt8, &ptr);
+    }
+#if SIZEOF_LONG != 8 
+    dbQueryExpression& operator,(long const& ptr) {
+        return add(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &ptr);
+    }
+    dbQueryExpression& operator,(unsigned long const& ptr) {
+        return add(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &ptr);
+    }
+#endif
     dbQueryExpression& operator,(real4 const& ptr) {
         return add(dbQueryElement::qVarReal4, &ptr);
     }
@@ -180,7 +204,10 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
         return add(dbQueryElement::qVarReal8, &ptr);
     }
     dbQueryExpression& operator,(void const* ptr) { 
-	return add(dbQueryElement::qVarRawData, ptr);
+        return add(dbQueryElement::qVarRawData, ptr);
+    }
+    dbQueryExpression& operator,(void** ptr) { 
+        return add(dbQueryElement::qVarRawDataPtr, ptr);
     }
 #ifndef bool
     dbQueryExpression& operator,(bool const& ptr) {
@@ -189,7 +216,7 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
 #endif
     dbQueryExpression& operator,(char_t const* ptr) {
         return add(operand ? dbQueryElement::qVarString
-		   : dbQueryElement::qExpression, ptr);
+                   : dbQueryElement::qExpression, ptr);
     }
     dbQueryExpression& operator,(char_t const** ptr) {
         return add(dbQueryElement::qVarStringPtr, ptr);
@@ -200,55 +227,81 @@ class GIGABASE_DLL_ENTRY dbQueryExpression {
     dbQueryExpression& operator,(rectangle const& rect) {
         return add(dbQueryElement::qVarRectangle, &rect);
     }
+    dbQueryExpression& operator,(rectangle* rect) {
+        return add(dbQueryElement::qVarRectanglePtr, &rect);
+    }
 #ifdef USE_STD_STRING
     dbQueryExpression& operator,(STD_STRING const& str) {
         return add(dbQueryElement::qVarStdString, &str);
     }
 #endif
+#ifdef USE_MFC_STRING
+    dbQueryExpression& operator,(MFC_STRING const& str) {
+        return add(dbQueryElement::qVarMfcString, &str);
+    }
+#endif
 
     dbQueryExpression& operator,(dbQueryExpression const& expr) {
-	*last = new dbQueryElement(dbQueryElement::qExpression, "(");
-	(*last)->next = expr.first;
-	last = expr.last;
-	*last = new dbQueryElement(dbQueryElement::qExpression, ")");
-	last = &(*last)->next;
-	operand = false;
-	return *this;
+        *last = new dbQueryElement(dbQueryElement::qExpression, _T("("));
+        (*last)->next = expr.first;
+        last = expr.last;
+        *last = new dbQueryElement(dbQueryElement::qExpression, _T(")"));
+        last = &(*last)->next;
+        operand = false;
+        return *this;
     }
     dbQueryExpression& operator,(dbComponent const& comp) {
-	add(dbQueryElement::qExpression, comp.structure);
-	if (comp.field != NULL) {
-	    add(dbQueryElement::qExpression, ".");
-	    add(dbQueryElement::qExpression, comp.field);
-	}
-	operand = false;
-	return *this;
+        add(dbQueryElement::qExpression, comp.structure);
+        if (comp.field != NULL) {
+            add(dbQueryElement::qExpression, _T("."));
+            add(dbQueryElement::qExpression, comp.field);
+        }
+        operand = false;
+        return *this;
     }
     dbQueryExpression& operator += (dbComponent const& comp) { 
         return *this,comp;
     }
     dbQueryExpression& operator += (char const* ptr) { 
-	return add(dbQueryElement::qExpression, ptr);
+        return add(dbQueryElement::qExpression, ptr);
     }
 #ifndef NO_MEMBER_TEMPLATES
     template<class T>
     dbQueryExpression& operator,(dbReference<T> const& value) { 
-	return add(dbQueryElement::qVarReference, &value, &T::dbDescriptor);
+        return add(dbQueryElement::qVarReference, &value, &T::dbDescriptor);
     }
 
     template<class T>
     inline dbQueryExpression& operator,(dbArray< dbReference<T> > const& value) { 
-	return add(dbQueryElement::qVarArrayOfRef, &value, 
-		      &T::dbDescriptor);
+        return add(dbQueryElement::qVarArrayOfRef, &value, 
+                      &T::dbDescriptor);
     }
 
     template<class T>
     inline dbQueryExpression& operator,(dbArray< dbReference<T> >const* const& value) { 
- 	return add(dbQueryElement::qVarArrayOfRefPtr, &value, &T::dbDescriptor);
+        return add(dbQueryElement::qVarArrayOfRefPtr, &value, &T::dbDescriptor);
+    }
+#endif
+#if !defined(_MSC_VER) || _MSC_VER+0 >= 1300
+    inline dbQueryExpression& operator,(dbArray<db_int4> const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt4, &value);
+    }
+
+    inline dbQueryExpression& operator,(dbArray<db_int4>const* const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt4Ptr, &value);
+    }
+
+    inline dbQueryExpression& operator,(dbArray<db_int8> const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt8, &value);
+    }
+
+    inline dbQueryExpression& operator,(dbArray<db_int8>const* const& value) { 
+        return add(dbQueryElement::qVarArrayOfInt8Ptr, &value);
     }
 #endif
 };
 
+class dbExprNode;
 class dbOrderByNode;
 class dbFollowByNode;
 
@@ -261,11 +314,13 @@ class GIGABASE_DLL_ENTRY dbCompiledQuery {
     dbOrderByNode*     order;
     dbFollowByNode*    follow;
     dbTableDescriptor* table;
+    int                schemeVersion;
 
-    size_t             stmtLimitStart;
-    size_t             stmtLimitLen;
-    int4*              stmtLimitStartPtr;
-    int4*              stmtLimitLenPtr;
+    cardinality_t  
+    stmtLimitStart;
+    cardinality_t      stmtLimitLen;
+    cardinality_t*     stmtLimitStartPtr;
+    cardinality_t*     stmtLimitLenPtr;
     bool               limitSpecified;
 
     enum IteratorInit {
@@ -308,7 +363,9 @@ class GIGABASE_DLL_ENTRY dbCompiledQuery {
 class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     friend class dbCompiler;
     friend class dbDatabase;
+    friend class dbSession;
     friend class dbSubSql;
+    friend class dbCLI;
   private:
     dbMutex            mutex;
     dbQueryElement*    elements;
@@ -318,22 +375,24 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     //
     // Prohibite query copying
     //
-    dbQuery(dbQuery const&) {}
+    dbQuery(dbQuery const&) : dbCompiledQuery() {}
     dbQuery& operator =(dbQuery const&) { return *this; }
 
   public:
     int                pos; // position of condition in statement
 
-    char* dump(char* buf) { 
-        char* p = buf;
+    bool compileError() { return !isEmpty() && !compiled(); }
+
+    char_t* dump(char_t* buf) { 
+        char_t* p = buf;
         for (dbQueryElement* elem = elements; elem != NULL; elem = elem->next) { 
             p = elem->dump(p);
         }
         return buf;
     }
 
-    char* dumpValues(char* buf) { 
-        char* p = buf;
+    char_t* dumpValues(char_t* buf) { 
+        char_t* p = buf;
         for (dbQueryElement* elem = elements; elem != NULL; elem = elem->next) { 
             p = elem->dumpValues(p);
         }
@@ -348,6 +407,12 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
         return *this;
     }
 
+    bool isEmpty() { 
+        return elements == NULL 
+            || (elements->next == NULL && elements->type == dbQueryElement::qExpression && *(char_t*)elements->ptr == 0);
+    }
+
+
     dbQuery& reset();
 
     //
@@ -367,16 +432,30 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
 
     dbQuery& add(dbQueryExpression const& expr);
 
+    dbQuery& And(dbQueryExpression const& expr) { 
+        if (elements != NULL) { 
+            append(dbQueryElement::qExpression, _T("and"));
+        }
+        return add(expr);
+    }
+
+    dbQuery& Or(dbQueryExpression const& expr) { 
+        if (elements != NULL) { 
+            append(dbQueryElement::qExpression, _T("or"));
+        }
+        return add(expr);
+    }
+
     dbQuery& And(char_t const* str) {
         if (elements != NULL) {
-            append(dbQueryElement::qExpression, "and");
+            append(dbQueryElement::qExpression, _T("and"));
         }
         return append(dbQueryElement::qExpression, str);
     }
 
     dbQuery& Or(char_t const* str) {
         if (elements != NULL) {
-            append(dbQueryElement::qExpression, "or");
+            append(dbQueryElement::qExpression, _T("or"));
         }
         return append(dbQueryElement::qExpression, str);
     }
@@ -394,11 +473,20 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& add(rectangle const& rect) {
         return append(dbQueryElement::qVarRectangle, &rect);
     }
+    dbQuery& add(rectangle* rect) {
+        return append(dbQueryElement::qVarRectanglePtr, &rect);
+    }
 #ifdef USE_STD_STRING
     dbQuery& add(STD_STRING const& str) { 
         return append(dbQueryElement::qVarStdString, &str);
     }
     dbQuery& operator,(STD_STRING const& str) { return add(str); }
+#endif
+#ifdef USE_MFC_STRING
+    dbQuery& add(MFC_STRING const& str) { 
+        return append(dbQueryElement::qVarMfcString, &str);
+    }
+    dbQuery& operator,(MFC_STRING const& str) { return add(str); }
 #endif
     dbQuery& add(int1 const& value) {
         return append(dbQueryElement::qVarInt1, &value);
@@ -412,6 +500,26 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& add (db_int8 const& value) {
         return append(dbQueryElement::qVarInt8, &value);
     }
+    dbQuery& add(nat1 const& value) {
+        return append(dbQueryElement::qVarInt1, &value);
+    }
+    dbQuery& add (nat2 const& value) {
+        return append(dbQueryElement::qVarInt2, &value);
+    }
+    dbQuery& add (nat4 const& value) {
+        return append(dbQueryElement::qVarInt4, &value);
+    }
+    dbQuery& add (db_nat8 const& value) {
+        return append(dbQueryElement::qVarInt8, &value);
+    }
+#if SIZEOF_LONG != 8 
+    dbQuery& add (long const& value) {
+        return append(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &value);
+    }
+    dbQuery& add (unsigned long const& value) {
+        return append(sizeof(long) == 4 ? dbQueryElement::qVarInt4 : dbQueryElement::qVarInt8, &value);
+    }
+#endif
     dbQuery& add (real4 const& value) {
         return append(dbQueryElement::qVarReal4, &value);
     }
@@ -421,6 +529,9 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& add(void const* value) { 
         return append(dbQueryElement::qVarRawData, value);
     }
+    dbQuery& add(void** value) { 
+        return append(dbQueryElement::qVarRawDataPtr, value);
+    }
 #ifndef bool
     dbQuery& add(bool const& value) {
         return append(dbQueryElement::qVarBool, &value);
@@ -435,6 +546,14 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
     dbQuery& operator,(int2 const&  value) { return add(value); }
     dbQuery& operator,(int4 const&  value) { return add(value); }
     dbQuery& operator,(db_int8 const&  value) { return add(value); }
+    dbQuery& operator,(nat1 const&  value) { return add(value); }
+    dbQuery& operator,(nat2 const&  value) { return add(value); }
+    dbQuery& operator,(nat4 const&  value) { return add(value); }
+    dbQuery& operator,(db_nat8 const&  value) { return add(value); }
+#if SIZEOF_LONG != 8
+    dbQuery& operator,(long const&  value) { return add(value); }
+    dbQuery& operator,(unsigned long const&  value) { return add(value); }
+#endif
     dbQuery& operator,(real4 const& value) { return add(value); }
     dbQuery& operator,(real8 const& value) { return add(value); }
     dbQuery& operator,(void const*  value) { return add(value); }
@@ -445,6 +564,40 @@ class GIGABASE_DLL_ENTRY dbQuery : public dbCompiledQuery {
         return reset().append(dbQueryElement::qExpression, str);
     }
 
+#if !defined(_MSC_VER) || _MSC_VER+0 >= 1300
+    inline dbQuery& operator,(dbArray<db_int4> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4, &value);
+    }
+
+    inline dbQuery& operator,(dbArray<db_int4>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4Ptr, &value);
+    }
+
+    inline dbQuery& operator,(dbArray<db_int8> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8, &value);
+    }
+
+    inline dbQuery& operator,(dbArray<db_int8>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8Ptr, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int4> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int4>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt4Ptr, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int8> const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8, &value);
+    }
+
+    inline dbQuery& add(dbArray<db_int8>const* const& value) { 
+        return append(dbQueryElement::qVarArrayOfInt8Ptr, &value);
+    }
+#endif
+
 #ifndef NO_MEMBER_TEMPLATES
     template<class T>
     dbQuery& operator,(dbReference<T> const& value) { 
@@ -514,8 +667,7 @@ inline dbQueryExpression& operator,(dbQueryExpression& expr, dbReference<T> cons
 }
 template<class T>
 inline dbQueryExpression& operator,(dbQueryExpression& expr, dbArray< dbReference<T> > const& value) { 
-    return expr.add(dbQueryElement::qVarArrayOfRef, &value, 
-                    &T::dbDescriptor);
+    return expr.add(dbQueryElement::qVarArrayOfRef, &value, &T::dbDescriptor);
 }
 
 template<class T>
@@ -525,24 +677,20 @@ inline dbQueryExpression& operator,(dbQueryExpression& expr, dbArray< dbReferenc
 
 template<class T>
 inline dbQuery& operator,(dbQuery& query, dbReference<T> const& value) { 
-    return query.append(dbQueryElement::qVarReference, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarReference, &value, &T::dbDescriptor);
 }
 
 template<class T>
-inline dbQuery& operator,(dbQuery& query, 
-                             dbArray< dbReference<T> > const& value) 
+inline dbQuery& operator,(dbQuery& query, dbArray< dbReference<T> > const& value) 
 { 
-    return query.append(dbQueryElement::qVarArrayOfRef, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarArrayOfRef, &value, &T::dbDescriptor);
 }
 
 template<class T>
 inline dbQuery& operator,(dbQuery& query, 
                           dbArray< dbReference<T> >const* const& value) 
 { 
-    return query.append(dbQueryElement::qVarArrayOfRefPtr, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarArrayOfRefPtr, &value, &T::dbDescriptor);
 }
 
 template<class T>
@@ -552,8 +700,7 @@ inline dbQuery& add(dbQuery& query, dbReference<T> const& value) {
 
 template<class T>
 inline dbQuery& add(dbQuery& query, dbArray< dbReference<T> > const& value) { 
-    return query.append(dbQueryElement::qVarArrayOfRef, &value, 
-                        &T::dbDescriptor);
+    return query.append(dbQueryElement::qVarArrayOfRef, &value, &T::dbDescriptor);
 }
 
 template<class T>
@@ -563,6 +710,7 @@ inline dbQuery& add(dbQuery& query, dbArray< dbReference<T> >const* const& value
 #endif
 
 #define USER_FUNC(f) static dbUserFunction f##_descriptor(&f, STRLITERAL(#f))
+#define USER_FUNC2(f,g) static dbUserFunction f##_descriptor(g, STRLITERAL(#f))
 
 class  dbInheritedAttribute;
 class  dbSynthesizedAttribute;
@@ -584,7 +732,7 @@ class GIGABASE_DLL_ENTRY  dbUserFunctionArgument {
     dbArgumentType type; 
     union {
         real8         realValue;
-        db_int8          intValue;
+        db_int8       intValue;
         bool          boolValue;
         char_t const* strValue;
         oid_t         oidValue;
@@ -612,7 +760,7 @@ class GIGABASE_DLL_ENTRY dbUserFunction {
     friend class dbCompiler;
 
     void*   fptr;
-    char_t* name;
+    char_t* fname;
 
     dbUserFunction* next;
     static dbUserFunction* list;
@@ -651,7 +799,7 @@ class GIGABASE_DLL_ENTRY dbUserFunction {
 
     static dbUserFunction* find(char_t const* name) {
         for (dbUserFunction* func = list; func != NULL; func = func->next) {
-            if (name == func->name) {
+            if (name == func->fname) {
                 return func;
             }
         }
@@ -738,15 +886,11 @@ class GIGABASE_DLL_ENTRY dbUserFunction {
     dbUserFunction(real8 (__cdecl *f)(dbUserFunctionArgument&, dbUserFunctionArgument&, dbUserFunctionArgument&), char_t* name) { 
         bind(name, (void*)f, fArgArgArg2Real); 
     }
+
+    ~dbUserFunction();
 };
 
 END_GIGABASE_NAMESPACE
 
 
 #endif
-
-
-
-
-
-
