@@ -1,4 +1,4 @@
-//-< CRECTANGLE.H >--------------------------------------------------*--------*
+//-< RECTANGLE.H >---------------------------------------------------*--------*
 // GigaBASE                  Version 1.0         (c) 1999  GARRET    *     ?  *
 // (Post Relational Database Management System)                      *   /\|  *
 //                                                                   *  /  \  *
@@ -9,7 +9,7 @@
 //-------------------------------------------------------------------*--------*
 
 #ifndef __RECTANGLE_H__
-#define __RECTANLGE_H__
+#define __RECTANGLE_H__
 
 BEGIN_GIGABASE_NAMESPACE
 
@@ -19,10 +19,12 @@ BEGIN_GIGABASE_NAMESPACE
 
 #ifndef RECTANGLE_COORDINATE_TYPE
 #define RECTANGLE_COORDINATE_TYPE int4
+#define RECTANGLE_AREA_TYPE int8
 #endif
 
 
 typedef RECTANGLE_COORDINATE_TYPE coord_t;
+typedef RECTANGLE_AREA_TYPE       area_t;
 
 /**
  * Multidimensional rectangle
@@ -40,15 +42,15 @@ class GIGABASE_DLL_ENTRY rectangle
     coord_t boundary[dim*2];
 
     /**
-     * Calculate dkistance between two rectangles
+     * Calculate distance between two rectangles
      */
     friend coord_t GIGABASE_DLL_ENTRY distance(rectangle const& r, rectangle const& q);
 
     /**
      * Caclulate area of rectangle
      */
-    friend coord_t area(rectangle const& r) { 
-        coord_t area = 1;
+    friend area_t area(rectangle const& r) { 
+        area_t area = 1;
         for (int i = dim; --i >= 0; area *= r.boundary[i+dim] - r.boundary[i]);
         return area;
     }
