@@ -70,7 +70,7 @@ class GIGABASE_DLL_ENTRY dbNullReference {};
  */
 extern GIGABASE_DLL_ENTRY dbNullReference null;
 
-#if defined(_MSC_VER) && _MSC_VER+0 <= 1100 || defined(__MWERKS__)
+#if (defined(_MSC_VER) && (_MSC_VER+0 < 1200 || _MSC_VER >= 1310)) || defined(__MWERKS__)
 //
 // Visual C++ prior to 5.0 version (with applied Service Pack 3)
 // didn't support lazy template instantiation. As far as VC has bug
@@ -89,10 +89,10 @@ class dbReference : public dbAnyReference {
   public:
     dbFieldDescriptor* dbDescribeComponents(dbFieldDescriptor* fd) {
         fd->type = fd->appType = dbField::tpReference;
-#if defined(_MSC_VER) && _MSC_VER+0 <= 1100 || defined(__MWERKS__)
+#if defined(_MSC_VER) && (_MSC_VER+0 < 1200 || _MSC_VER >= 1310) || defined(__MWERKS__)
         fd->refTable = dbGetTableDescriptor((T*)0);
 #else
-#if (defined(__GNUC__) && __GNUC_MINOR__ <= 95) || defined(__VACPP_MULTI__) || defined(__IBMCPP__)
+#if GNUC_BEFORE(2,96) || defined(__VACPP_MULTI__) || defined(__IBMCPP__) || defined(HPUX_ITANIUM64)
         extern dbTableDescriptor* dbGetTableDescriptor(T*);
         fd->refTable = dbGetTableDescriptor((T*)0);
 #else
@@ -106,7 +106,7 @@ class dbReference : public dbAnyReference {
 
     /**
      * Assignment operator
-     * @param assigned reference of the same type
+     * @param ref assigned reference of the same type
      * @return this reference
      */
     dbReference<T>& operator = (dbReference<T> const& ref) {
@@ -125,7 +125,7 @@ class dbReference : public dbAnyReference {
 
     /**
      * Unsafe assignment operator. Assign any refernce.      
-     * @param assigned reference. If it is not of the same type - result is unpredicted.
+     * @param ref assigned reference. If it is not of the same type - result is unpredicted.
      * @return this reference
      */
     dbReference<T>& unsafeAssign(dbAnyReference const& ref) {
