@@ -11,24 +11,34 @@
 #ifndef __SYNC_H__
 #define __SYNC_H__
 
-
 BEGIN_GIGABASE_NAMESPACE
 
-#if defined(_WIN32)
+#if defined(_WIN32) && !defined(__SYMBIAN32__)
+
 class GIGABASE_DLL_ENTRY dbMutex {
-    CRITICAL_SECTION cs;
+    CRITICAL_SECTION     cs;
+    bool                 initialized;
   public:
     dbMutex() {
         InitializeCriticalSection(&cs);
+        initialized = true;
     }
     ~dbMutex() {
         DeleteCriticalSection(&cs);
+        initialized = false;
+    }
+    bool isInitialized() { 
+        return initialized;
     }
     void lock() {
-        EnterCriticalSection(&cs);
+        if (initialized) { 
+            EnterCriticalSection(&cs);
+        }
     }
     void unlock() {
-        LeaveCriticalSection(&cs);
+        if (initialized) { 
+            LeaveCriticalSection(&cs);
+        }
     }
 };
 
@@ -46,12 +56,15 @@ class GIGABASE_DLL_ENTRY dbThread {
         SetThreadPriority(h, pri == THR_PRI_LOW ? THREAD_PRIORITY_IDLE : THREAD_PRIORITY_HIGHEST);
     }
 
+    static void sleep(time_t sec) { 
+        Sleep((DWORD)(sec*1000));
+    }
+
     typedef void (thread_proc* thread_proc_t)(void*);
 
     void create(thread_proc_t f, void* arg) {
         DWORD threadid;
-        h = CreateThread(NULL, NULL, LPTHREAD_START_ROUTINE(f), arg,
-                         0, &threadid);
+        h = CreateThread(NULL, 0, LPTHREAD_START_ROUTINE(f), arg, 0, &threadid);
     }
     void join() {
         WaitForSingleObject(h, INFINITE);
@@ -87,7 +100,7 @@ class GIGABASE_DLL_ENTRY dbSemaphore {
   public:
     void wait(dbMutex& mutex, time_t timeout = INFINITE) {
         mutex.unlock();
-        int rc = WaitForSingleObject(s, timeout == INFINITE ? timeout : timeout*1000);
+        int rc = WaitForSingleObject(s, (DWORD)(timeout == (time_t)INFINITE ? timeout : timeout*1000));
         assert(rc == WAIT_OBJECT_0 || rc == WAIT_TIMEOUT);
         mutex.lock();
     }
@@ -102,6 +115,9 @@ class GIGABASE_DLL_ENTRY dbSemaphore {
     }
     void close() {
         CloseHandle(s);
+    } 
+    dbSemaphore() { 
+        s = NULL;
     }
 };
 
@@ -113,7 +129,7 @@ class GIGABASE_DLL_ENTRY dbEvent {
     void wait(dbMutex& mutex, time_t timeout = INFINITE) {
         nWaitingThreads += 1;
         mutex.unlock();
-        int rc = WaitForSingleObject(e, timeout == INFINITE ? timeout : timeout*1000);
+        int rc = WaitForSingleObject(e, (DWORD)(timeout == (time_t)INFINITE ? timeout : timeout*1000));
         assert(rc == WAIT_OBJECT_0 || rc == WAIT_TIMEOUT);
         mutex.lock();
         nWaitingThreads -= 1;
@@ -142,6 +158,9 @@ class GIGABASE_DLL_ENTRY dbEvent {
     void close() {
         CloseHandle(e);
     }
+    dbEvent() { 
+        e = NULL;
+    }
 };
 
 template<class T>
@@ -156,7 +175,7 @@ class dbThreadContext {
     }
     dbThreadContext() {
         index = TlsAlloc();
-        assert(index != TLS_OUT_OF_INDEXES);
+        assert(index != (int)TLS_OUT_OF_INDEXES);
     }
     ~dbThreadContext() {
         TlsFree(index);
@@ -167,7 +186,280 @@ class dbThreadContext {
 
 #define thread_proc
 
-#ifndef NO_PTHREADS
+#if defined(NO_PTHREADS)
+
+class dbMutex {
+    bool initialized;
+
+   public:
+    dbMutex() {
+        initialized = true;
+    }
+
+    ~dbMutex() { 
+        initialized = false;
+    }
+
+    bool isInitialized() { 
+        return initialized;
+    }
+
+    void lock() {}
+    void unlock() {}
+};
+
+class dbThread {
+  public:
+    typedef void (thread_proc* thread_proc_t)(void*);
+    void create(thread_proc_t f, void* arg) { f(arg); }
+    void join() {}
+    void detach() {}
+    
+    enum ThreadPriority { 
+        THR_PRI_LOW, 
+        THR_PRI_HIGH
+    };
+    void setPriority(ThreadPriority) {}
+
+    static int numberOfProcessors() { return 1; }
+};
+
+class dbSemaphore {
+    int count;
+  public:
+    void wait(dbMutex&, time_t=0) {
+        assert (count > 0);
+        count -= 1;
+    }
+    void signal(unsigned inc = 1) {
+        count += inc;
+    }
+    void open(unsigned initValue = 0) {
+        count = initValue;
+    }
+    void close() {}
+};
+
+class dbEvent {
+    bool signaled;
+  public:
+    void wait(dbMutex&, time_t=0) {
+        assert(signaled);
+    }
+    void signal() {
+        signaled = true;
+    }
+    void reset() {
+        signaled = false;
+    }
+    void open(bool initValue = false) {
+        signaled = initValue;
+    }
+    void pulse() {}
+    void close() {}
+};
+
+template<class T>
+class dbThreadContext {
+    T* value;
+  public:
+    T* get() {
+        return value;
+    }
+    void set(T* value) {
+        this->value = value;
+    }
+    dbThreadContext() { value = NULL; }
+};
+
+#elif defined(__SYMBIAN32__)
+
+class dbMutex 
+{
+    friend class dbEvent;
+    friend class dbSemaphore;
+    RMutex cs;
+    bool   initialized;
+  public:
+    dbMutex() {
+        int rc = cs.CreateLocal();
+        assert(rc == KErrNone);
+        initialized = true;
+    }
+    ~dbMutex() {
+        cs.Close();
+        initialized = false;
+    }
+    bool isInitialized() { 
+        return initialized;
+    }
+    void lock() {
+        if (initialized) { 
+            cs.Wait();
+        }
+    }
+    void unlock() {
+        if (initialized) { 
+            cs.Signal();
+        }
+    }
+};    
+    
+class dbThread 
+{
+    RThread thread;
+    static TInt count;
+  public:
+    static size_t heapMinSize;
+    static size_t heapMaxSize;
+    static size_t stackSize;
+    
+    typedef void (thread_proc* thread_proc_t)(void*);
+    void create(thread_proc_t f, void* arg) { 
+        TBuf<16> name;
+        name.Format(_L("%d"), User::LockedInc(count));
+        int rc = thread.Create(name, (TThreadFunction)f, stackSize, heapMinSize, heapMaxSize, arg);
+        assert(rc == KErrNone);
+    }
+
+    void join() {
+        TRequestStatus status;
+        thread.Rendezvous(status);
+        status.Int();        
+    }
+    void detach() {}
+    
+    enum ThreadPriority { 
+        THR_PRI_LOW, 
+        THR_PRI_HIGH
+    };
+
+    void setPriority(ThreadPriority pri) {
+        thread.SetPriority(pri == THR_PRI_LOW ? EPriorityLess : EPriorityMore);
+    }
+
+    static int numberOfProcessors() { return 1; }
+};
+
+class dbEvent 
+{
+    RCondVar cond;
+    int      signaled;
+    long     n_signals;
+
+  public:
+    void wait(dbMutex& mutex) {
+        long before_n_signals = n_signals;
+        while (!signaled && n_signals == before_n_signals) {
+            int rc = cond.Wait(mutex.cs);
+            assert(rc == KErrNone);
+        }
+    }
+
+    bool wait(dbMutex& mutex, time_t timeout) {
+        if (!signaled) {
+            long before_n_signals = n_signals;
+            do { 
+                int rc = cond.TimedWait(mutex.cs, timeout*1000000);
+                if (rc == KErrTimedOut) { 
+                    return false;
+                }
+                assert(rc == KErrNone);
+            } while (!signaled && n_signals == before_n_signals);
+        }
+        return true;
+    }
+
+    void signal() {
+        signaled = true; 
+        n_signals += 1;
+        cond.Broadcast();
+    }
+
+    void pulse() { 
+        n_signals += 1;
+        cond.Broadcast();
+    }
+        
+    void reset() {
+        signaled = false;
+    }
+    void open(bool initValue = false) {
+        signaled = initValue;
+        n_signals = 0;
+        int rc = cond.CreateLocal();
+        assert(rc == KErrNone);
+    }
+    void close() {
+        cond.Close();
+    }
+};
+
+class dbSemaphore {
+    RCondVar cond;
+    int      count;
+  public:
+    void wait(dbMutex& mutex) {
+        while (count == 0) {
+            int rc = cond.Wait(mutex.cs);
+            assert(rc == KErrNone);
+        }
+        count -= 1;
+    }
+
+    bool wait(dbMutex& mutex, time_t timeout) {
+        if (count == 0) { 
+            do { 
+                int rc = cond.TimedWait(mutex.cs, timeout*1000000);
+                if (rc == KErrTimedOut) { 
+                    return false;
+                }
+                assert(rc == KErrNone);
+            } while (count == 0);
+        }
+        count -= 1;
+        return true;
+    }
+
+    void signal(unsigned inc = 1) {
+        count += inc;
+        if (inc > 1) {
+            cond.Broadcast();
+        } else if (inc == 1) {
+            cond.Signal();
+        }
+    }
+    void open(unsigned initValue = 0) {
+        int rc = cond.CreateLocal();
+        assert(rc == KErrNone);
+        count = initValue;
+    }
+    void close() {
+        cond.Close();
+    }
+};
+
+extern int dbTlsIndex;
+
+template<class T>
+class dbThreadContext {
+    int index;
+  public:
+    T* get() {
+        return (T*)UserSvr::DllTls(index);
+    }
+    void set(T* value) {
+        UserSvr::DllSetTls(index, value);
+    }
+    dbThreadContext() { 
+        index = ++dbTlsIndex;
+    }
+    ~dbThreadContext() { 
+        UserSvr::DllFreeTls(index);
+    }
+};
+
+#else // Posix threads
 
 END_GIGABASE_NAMESPACE
 
@@ -181,18 +473,28 @@ class dbMutex {
     friend class dbEvent;
     friend class dbSemaphore;
     pthread_mutex_t cs;
+    bool            initialized;
   public:
     dbMutex() {
         pthread_mutex_init(&cs, NULL);
+        initialized = true;
     }
     ~dbMutex() {
         pthread_mutex_destroy(&cs);
+        initialized = false;
+    }
+    bool isInitialized() { 
+        return initialized;
     }
     void lock() {
-        pthread_mutex_lock(&cs);
+        if (initialized) { 
+            pthread_mutex_lock(&cs);
+        }
     }
     void unlock() {
-        pthread_mutex_unlock(&cs);
+        if (initialized) { 
+            pthread_mutex_unlock(&cs);
+        }
     }
 };
 
@@ -222,15 +524,19 @@ class dbThread {
         THR_PRI_LOW, 
         THR_PRI_HIGH
     };
-    void setPriority(ThreadPriority pri) { 
 #if defined(PRI_OTHER_MIN) && defined(PRI_OTHER_MAX)
+    void setPriority(ThreadPriority pri) { 
         struct sched_param sp;
-        sp.sched_priority = pri == THR_PRI_LOW ? IPRI_OTHER_MIN : PRI_OTHER_MAX;
+        sp.sched_priority = pri == THR_PRI_LOW ? PRI_OTHER_MIN : PRI_OTHER_MAX;
         pthread_setschedparam(thread, SCHED_OTHER, &sp); 
-#endif
     }
+#else
+    void setPriority(ThreadPriority) {}
+#endif
 
-
+    static void sleep(time_t sec) { 
+        ::sleep(sec);
+    }
 
     void join() {
         void* result;
@@ -251,7 +557,7 @@ END_GIGABASE_NAMESPACE
 #include <linux/smp.h>
 BEGIN_GIGABASE_NAMESPACE
 inline int dbThread::numberOfProcessors() { return smp_num_cpus; }
-#elif defined(__FreeBSD__) || defined(__bsdi__) || defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__bsdi__) || defined(__OpenBSD__) || defined(__NetBSD__)
 #if defined(__bsdi__) || defined(__OpenBSD__)
 END_GIGABASE_NAMESPACE
 #include <sys/param.h>
@@ -286,9 +592,8 @@ class dbEvent {
         }
     }
 
-    void wait(dbMutex& mutex, time_t timeout) {
-        long before_n_signals = n_signals;
-        while (!signaled && n_signals == before_n_signals) {
+    bool wait(dbMutex& mutex, time_t timeout) {
+        if (!signaled) {
             struct timespec abs_ts; 
 #ifdef PTHREAD_GET_EXPIRATION_NP
             struct timespec rel_ts; 
@@ -301,8 +606,15 @@ class dbEvent {
             abs_ts.tv_sec = cur_tv.tv_sec + timeout; 
             abs_ts.tv_nsec = cur_tv.tv_usec*1000;
 #endif
-            pthread_cond_timedwait(&cond, &mutex.cs, &abs_ts);
+            long before_n_signals = n_signals;
+            do { 
+                int rc = pthread_cond_timedwait(&cond, &mutex.cs, &abs_ts);
+                if (rc != 0) {
+                    return false;
+                }
+            } while (!signaled && n_signals == before_n_signals);
         }
+        return true;
     }
 
     void signal() {
@@ -321,6 +633,7 @@ class dbEvent {
     }
     void open(bool initValue = false) {
         signaled = initValue;
+        n_signals = 0;
         pthread_cond_init(&cond, NULL);
     }
     void close() {
@@ -339,8 +652,8 @@ class dbSemaphore {
         count -= 1;
     }
 
-    void wait(dbMutex& mutex, time_t timeout) {
-        while (count == 0) {
+    bool wait(dbMutex& mutex, time_t timeout) {
+        if (count == 0) { 
             struct timespec abs_ts; 
 #ifdef PTHREAD_GET_EXPIRATION_NP
             struct timespec rel_ts; 
@@ -353,9 +666,15 @@ class dbSemaphore {
             abs_ts.tv_sec = cur_tv.tv_sec + timeout; 
             abs_ts.tv_nsec = cur_tv.tv_usec*1000;
 #endif
-            pthread_cond_timedwait(&cond, &mutex.cs, &abs_ts);
+            do { 
+                int rc = pthread_cond_timedwait(&cond, &mutex.cs, &abs_ts);
+                if (rc != 0) { 
+                    return false;
+                }
+            } while (count == 0);
         }
         count -= 1;
+        return true;
     }
 
     void signal(unsigned inc = 1) {
@@ -393,78 +712,6 @@ class dbThreadContext {
     }
 };
 
-#else
-
-class dbMutex {
-   public:
-    void lock() {}
-    void unlock() {}
-};
-
-class dbThread {
-  public:
-    typedef void (thread_proc* thread_proc_t)(void*);
-    void create(thread_proc_t f, void* arg) { f(arg); }
-    void join() {}
-    void detach() {}
-    
-    enum ThreadPriority { 
-        THR_PRI_LOW, 
-        THR_PRI_HIGH
-    };
-    void setPriority(ThreadPriority);
-
-    static int numberOfProcessors() { return 1; }
-};
-
-class dbSemaphore {
-    int count;
-  public:
-    void wait(dbMutex&, time_t=0) {
-        assert (count > 0);
-        count -= 1;
-    }
-    void signal(unsigned inc = 1) {
-        count += inc;
-    }
-    void open(unsigned initValue = 0) {
-        count = initValue;
-    }
-    void close() {}
-};
-
-class dbEvent {
-    bool signaled;
-  public:
-    void wait(dbMutex&, time_t=0) {
-        assert(signaled);
-    }
-    void signal() {
-        signaled = true;
-    }
-    void reset() {
-        signaled = false;
-    }
-    void open(bool initValue = false) {
-        signaled = initValue;
-    }
-    void close() {}
-};
-
-template<class T>
-class dbThreadContext {
-    T* value;
-  public:
-    T* get() {
-        return value;
-    }
-    void set(T* value) {
-        this->value = value;
-    }
-    dbThreadContext() { value = NULL; }
-};
-
-
 #endif
 
 #endif
@@ -481,13 +728,14 @@ class GIGABASE_DLL_ENTRY dbCriticalSection {
     }
 };
 
-#define SMALL_BUF_SIZE 1024
+#define SMALL_BUF_SIZE 512
 
 template<class T>
 class dbSmallBuffer {
   protected:
-    T* buf;
-    T  smallBuf[SMALL_BUF_SIZE];
+    T      smallBuf[SMALL_BUF_SIZE];
+    T*     buf;
+    size_t used;
 
   public:
     dbSmallBuffer(size_t size) {
@@ -496,6 +744,22 @@ class dbSmallBuffer {
         } else {
             buf = smallBuf;
         }
+        used = size;
+    }
+
+    dbSmallBuffer() { 
+        used = 0;
+        buf = smallBuf;
+    }
+
+    void put(size_t size) { 
+        if (size > SMALL_BUF_SIZE && size > used) { 
+            if (buf != smallBuf) { 
+                delete[] buf;
+            }
+            buf = new T[size];
+            used = size;
+        }
     }
 
     operator T*() { return buf; }
@@ -508,6 +772,7 @@ class dbSmallBuffer {
     }
 };
 
+class dbThreadPool;
 
 class GIGABASE_DLL_ENTRY dbPooledThread { 
   private:
